[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Vault client library for Python

[![API Version](https://img.shields.io/badge/api-v3.14.6-blue)](https://developer.factset.com/api-catalog/vault-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.Vault)](https://pypi.org/project/fds.sdk.Vault/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

Allow clients to fetch Analytics through APIs.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 3.14.6
- SDK version: 2.0.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.Vault==2.0.0
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.Vault==2.0.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.Vault
from fds.sdk.Vault.api import accounts_api
from fds.sdk.Vault.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.Vault.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.Vault.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.Vault.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = accounts_api.AccountsApi(api_client)
    path = "Client:" # str | The directory to get the accounts and sub-directories in

    try:
        # Get accounts and sub-directories in a directory
        # example passing only required values which don't have defaults set
        api_response = api_instance.get_accounts(path)

        pprint(api_response)
    except fds.sdk.Vault.ApiException as e:
        print("Exception when calling AccountsApi->get_accounts: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Get accounts and sub-directories in a directory
    #     api_response, http_status_code, response_headers = api_instance.get_accounts_with_http_info(path)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Vault.ApiException as e:
    #     print("Exception when calling AccountsApi->get_accounts: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Get accounts and sub-directories in a directory
    #     async_result = api_instance.get_accounts_async(path)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.Vault.ApiException as e:
    #     print("Exception when calling AccountsApi->get_accounts: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Get accounts and sub-directories in a directory
    #     async_result = api_instance.get_accounts_with_http_info_async(path)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Vault.ApiException as e:
    #     print("Exception when calling AccountsApi->get_accounts: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.Vault

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.Vault.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.Vault

configuration = fds.sdk.Vault.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.Vault

configuration = fds.sdk.Vault.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.Vault

configuration = fds.sdk.Vault.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*AccountsApi* | [**get_accounts**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/AccountsApi.md#get_accounts) | **GET** /analytics/lookups/v3/accounts/{path} | Get accounts and sub-directories in a directory
*ComponentsApi* | [**get_vault_component_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ComponentsApi.md#get_vault_component_by_id) | **GET** /analytics/engines/vault/v3/components/{id} | Get Vault component by id
*ComponentsApi* | [**get_vault_components**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ComponentsApi.md#get_vault_components) | **GET** /analytics/engines/vault/v3/components | Get Vault components
*ConfigurationsApi* | [**get_vault_configuration_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ConfigurationsApi.md#get_vault_configuration_by_id) | **GET** /analytics/engines/vault/v3/configurations/{id} | Get Vault configuration by id
*ConfigurationsApi* | [**get_vault_configurations**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ConfigurationsApi.md#get_vault_configurations) | **GET** /analytics/engines/vault/v3/configurations | Get Vault configurations
*DatesApi* | [**convert_vault_dates_to_absolute_format**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/DatesApi.md#convert_vault_dates_to_absolute_format) | **GET** /analytics/engines/vault/v3/dates | Convert Vault dates to absolute format
*DocumentsApi* | [**get_vault_documents**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/DocumentsApi.md#get_vault_documents) | **GET** /analytics/engines/vault/v3/documents/{path} | Get Vault documents and sub-directories in a directory
*FrequenciesApi* | [**get_vault_frequencies**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/FrequenciesApi.md#get_vault_frequencies) | **GET** /analytics/engines/vault/v3/frequencies | Get Vault frequencies
*VaultCalculationsApi* | [**cancel_calculation_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultCalculationsApi.md#cancel_calculation_by_id) | **DELETE** /analytics/engines/vault/v3/calculations/{id} | Cancel Vault calculation by id
*VaultCalculationsApi* | [**get_all_calculations**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultCalculationsApi.md#get_all_calculations) | **GET** /analytics/engines/vault/v3/calculations | Get all calculations
*VaultCalculationsApi* | [**get_calculation_parameters**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultCalculationsApi.md#get_calculation_parameters) | **GET** /analytics/engines/vault/v3/calculations/{id} | Get Vault calculation parameters by id
*VaultCalculationsApi* | [**get_calculation_status_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultCalculationsApi.md#get_calculation_status_by_id) | **GET** /analytics/engines/vault/v3/calculations/{id}/status | Get Vault calculation status by id
*VaultCalculationsApi* | [**get_calculation_unit_result_by_id**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultCalculationsApi.md#get_calculation_unit_result_by_id) | **GET** /analytics/engines/vault/v3/calculations/{id}/units/{unitId}/result | Get Vault calculation result by id
*VaultCalculationsApi* | [**post_and_calculate**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultCalculationsApi.md#post_and_calculate) | **POST** /analytics/engines/vault/v3/calculations | Create and Run Vault calculation
*VaultCalculationsApi* | [**put_and_calculate**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultCalculationsApi.md#put_and_calculate) | **PUT** /analytics/engines/vault/v3/calculations/{id} | Create or Update Vault calculation and run it.


## Documentation For Models

 - [AccountDirectories](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/AccountDirectories.md)
 - [AccountDirectoriesRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/AccountDirectoriesRoot.md)
 - [CalculationMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/CalculationMeta.md)
 - [CalculationStatus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/CalculationStatus.md)
 - [CalculationStatusMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/CalculationStatusMeta.md)
 - [CalculationStatusRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/CalculationStatusRoot.md)
 - [CalculationUnitStatus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/CalculationUnitStatus.md)
 - [CalculationUnitStatusMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/CalculationUnitStatusMeta.md)
 - [CalculationsSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/CalculationsSummary.md)
 - [CalculationsSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/CalculationsSummaryRoot.md)
 - [ClientErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ClientErrorResponse.md)
 - [ComponentSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ComponentSummary.md)
 - [ComponentSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ComponentSummaryRoot.md)
 - [ConfigurationAccount](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ConfigurationAccount.md)
 - [DateParametersSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/DateParametersSummary.md)
 - [DateParametersSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/DateParametersSummaryRoot.md)
 - [DocumentDirectories](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/DocumentDirectories.md)
 - [DocumentDirectoriesRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/DocumentDirectoriesRoot.md)
 - [Error](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/Error.md)
 - [ErrorSource](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ErrorSource.md)
 - [Frequency](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/Frequency.md)
 - [FrequencyRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/FrequencyRoot.md)
 - [ObjectRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/ObjectRoot.md)
 - [PaginationInfo](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/PaginationInfo.md)
 - [PaginationMeta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/PaginationMeta.md)
 - [VaultCalculationParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultCalculationParameters.md)
 - [VaultCalculationParametersRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultCalculationParametersRoot.md)
 - [VaultComponent](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultComponent.md)
 - [VaultComponentRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultComponentRoot.md)
 - [VaultConfiguration](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultConfiguration.md)
 - [VaultConfigurationRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultConfigurationRoot.md)
 - [VaultConfigurationSummary](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultConfigurationSummary.md)
 - [VaultConfigurationSummaryRoot](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultConfigurationSummaryRoot.md)
 - [VaultDateParameters](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultDateParameters.md)
 - [VaultIdentifier](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Vault/v3/docs/VaultIdentifier.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.Vault.apis and fds.sdk.Vault.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.Vault.api.default_api import DefaultApi`
- `from fds.sdk.Vault.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.Vault
from fds.sdk.Vault.apis import *
from fds.sdk.Vault.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

